/*
 * File......: SNOW.PRG
 * Author....: Brian Dukes
 * Date......: $Date$
 * Revision..: $Revision$
 * Log file..: $Logfile$
 *
 * This is an original work by Brian Dukes and is placed in the public
 * domain.
 *
 * Modification history:
 * ---------------------
 *
 * $Log$
 *
 */


/*  $DOC$
 *  $FUNCNAME$
 *      Xmas_Saver()
 *  $CATEGORY$
 *      Screen Saver
 *  $ONELINER$
 *      Christmas / New Year and general purpose Snowfield Screen Saver module
 *  $SYNTAX$
 *      Xmas_Saver( [bExit] )
 *  $ARGUMENTS$
 *       bExit - Optional code block which is evaluated to see if
 *               the screen saver should exit, or continue running.
 *               defaults to {|| inkey() == 0 }, i.e, any key exits!
 *
 *
 *  $RETURNS$
 *      NIL
 *
 *  $DESCRIPTION$
 *      This function can be placed as a screen saver,  it detects the christmas
 *      period and the new year period itself, and puts up some seasonal greetings!
 *
 *
 *  $EXAMPLES$
 *      Xmas_Saver( {|| time() == "20:00:00" } )    // save the screen till 8pm!
 *      Xmas_Saver( {|| My_Inkey() != 0 } )         // save until a key is pressed!
 *  $END$
 */


function Xmas_Saver(bExit)
   local bQuit := if(bExit == NIL, {|| inkey() != 0 }, bExit)
   local aSnow := {}
   local i     := 0
   local x     := 25
   local y     := 1
   local delay := seconds()
   local c     := 1
   local aColour := {"W/N", "W+/N", "GR/N", "GR+/N", "R/N", "R+/N", "G/N", "G+/N", "BG/N", "BG+/N", "B/N", "B+/N", "RB/N", "RB+/N"}
   local lclr  := len(aColour)
   local nMaxFlakes := 20+GT_RAND1(80)
   local nDays := 0

   set colour to "W/N"
   setmode(50,80)
   clear screen
   aadd(aSnow,{})    // Plane 1  -- Furthest away, flakes move slower!
   aadd(aSnow,{})    // Plane 2  -- middle plane
   aadd(aSnow,{})    // Plane 3  -- Closest, fastest moving

   // initialise the snow flake array,  3 planes, with 25 flakes on each level.
   FOR i := 1 TO nMaxFlakes
      aadd(aSnow[1],{0,GT_RAND1(79),(GT_RAND1(10)+1)/10})
      aadd(aSnow[2],{0,GT_RAND1(79),(GT_RAND1(15)+1)/10})
      aadd(aSnow[3],{0,GT_RAND1(79),(GT_RAND1(20)+1)/10})
   NEXT

   x := GT_RAND1(39)
   y := GT_RAND1(15)
   delay := seconds()


   DO WHILE !eval(bQuit)
      dispbegin()
      clear screen
      FOR i := 1 TO nMaxFlakes
         @ aSnow[1][i][1], aSnow[1][i][2] say "" colour "W/N"
         @ aSnow[2][i][1], aSnow[2][i][2] say "" colour "W/N"
         @ aSnow[3][i][1], aSnow[3][i][2] say "*" colour "W+/N"
         aSnow[1][i][1] += aSnow[1][i][3]
         aSnow[2][i][1] += aSnow[2][i][3]
         aSnow[3][i][1] += aSnow[3][i][3]
         IF aSnow[1][i][1] > 50
            aSnow[1][i] := {0,GT_RAND1(79),(GT_RAND1(10)+1)/10}
         ENDIF
         IF aSnow[2][i][1] > 50
            aSnow[2][i] := {0,GT_RAND1(79),(GT_RAND1(15)+1)/10}
         ENDIF
         IF aSnow[3][i][1] > 50
            aSnow[3][i] := {0,GT_RAND1(79),(GT_RAND1(20)+1)/10}
         ENDIF
      NEXT


      IF strzero(month(date()),2)+strzero(day(date()),2) >= "1201" .AND. strzero(month(date()),2)+strzero(day(date()),2) <= "1231"
         nDays := ctod("25/12/"+alltrim(str(year(date()))))-date()
         @ x+ 0,y+11 say "          "                   colour aColour[c]
         @ x+ 1,y+11 say "          "                   colour aColour[c]
         @ x+ 2,y+11 say "          "                   colour aColour[c]
         @ x+ 3,y+11 say "              "                   colour aColour[c]
         @ x+ 4,y+11 say "          "                   colour aColour[c]

         @ x+ 6,y say "                  " colour aColour[c]
         @ x+ 7,y say "                    " colour aColour[c]
         @ x+ 8,y say "                   " colour aColour[c]
         @ x+ 9,y say "                         " colour aColour[c]
         @ x+10,y say "                     " colour aColour[c]
         IF nDays < 0
            nDays := (ctod("31/12/"+alltrim(str(year(date()))))+1)-date()
            @ x+12,y say "There are only "+alltrim(str(ndays))+" day"+if(nDays == 1,"","s")+" till New Year, Time Now is "+time() colour aColour[lClr-c+1]
         ELSE
            @ x+12,y say "There are only "+alltrim(str(ndays))+" day"+if(nDays == 1,"","s")+" till Christmas, Time Now is "+time() colour aColour[lClr-c+1]
         ENDIF

         @ 45,0 say ""              colour "W+/N"
         @ 46,0 say ""           colour "W+/N"
         @ 47,0 say ""         colour "W+/N"
         @ 48,0 say ""       colour "W+/N"
         @ 49,0 say replicate("",80) colour "W+/N"
         @ 48,65 say ""       colour "W+/N"
         @ 47,70 say ""       colour "W+/N"
      ELSE
         IF strzero(month(date()),2)+strzero(day(date()),2) >= "0101" .AND. strzero(month(date()),2)+strzero(day(date()),2) <= "0105"
                 @ x+ 0,y+06 say "          "                   colour aColour[c]
                 @ x+ 1,y+06 say "         "                   colour aColour[c]
                 @ x+ 2,y+06 say "       "                   colour aColour[c]
                 @ x+ 3,y+06 say "               "                   colour aColour[c]
                 @ x+ 4,y+06 say "             "                   colour aColour[c]

            @ x+ 6,y say "                " colour aColour[c]
            @ x+ 7,y say "                     " colour aColour[c]
            @ x+ 8,y say "                " colour aColour[c]
            @ x+ 9,y say "                     " colour aColour[c]
            @ x+10,y say "                  " colour aColour[c]
            @ x+12,y+13 say "Time Now is "+time() colour aColour[lClr-c+1]

            @ 42,0 say ""              colour "W+/N"
            @ 43,0 say ""              colour "W+/N"
            @ 44,0 say ""              colour "W+/N"
            @ 45,0 say ""           colour "W+/N"
            @ 46,0 say ""         colour "W+/N"
            @ 47,0 say ""       colour "W+/N"
            @ 48,0 say replicate("",80) colour "W+/N"
            @ 49,0 say replicate("",80) colour "W+/N"
            @ 45,65 say "      "       colour "W+/N"
            @ 46,65 say "    "       colour "W+/N"
            @ 47,65 say ""       colour "W+/N"
         ELSE
            GT_BigTime(x,y,aColour[c])
         ENDIF

      ENDIF
      c++
      IF c > lclr
         c := 1
      ENDIF

      IF seconds() - delay > 10
         x := GT_RAND1(39)
         y := GT_RAND1(15)
         delay := seconds()
      ENDIF

      dispend()
   ENDDO
return NIL
